/*
 *	Simple Linux Clustering Extensions -- Slave Daemon
 *
 *	Copyright (c) 1999 Martin Mares <mj@ucw.cz>
 *		  (c) 1999 Michal Jezek <mjez6230@ss1000.ms.mff.cuni.cz>
 *		  (c) 1999 Jan Lana <jlan3028@ss1000.ms.mff.cuni.cz>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>

#include "lib.h"

void
handle_timer(void)
{
  ALLOCA_MSG(mm, m, slice_status);

  (void) m;
  mk_status(mm);
  sm_send(mm);
}

static void
connect_to_master(char *name)
{
  struct hostent *he;
  int len = sizeof(struct in_addr) * SLICE_MAX_NODES;
  ALLOCA_MSG_BUF(m, n, in_addr, len);
  int i;

  DBG("Connecting to master node...\n");
  if (!(he = gethostbyname(name)))
    die("Unable to resolve %s: %m", name);
  define_node(0, (struct in_addr *) he->h_addr, 0);
  bzero(m, sizeof(struct slicemsg));
  m->from = 0xffff;
  m->cmd = SLICE_CMD_INIT;
  DBG("Sending initialization packet...\n");
  sm_send(m);
  DBG("Waiting for node table...\n");
  for(;;)
    {
      m->length = len;
      if (slicectl(SLICE_OP_GET_PACKET, m) < 0)
	die("get_packet: %m");
      if (!m->from && m->cmd == REPLY_TO(SLICE_CMD_INIT))
	break;
      if (debug)
	pkt_dump(m);
      printf("SLICE: Received unknown packet during initialization: %04x from %d\n", m->cmd, m->length);
    }
  this_node = m->to;
  DBG("Got node table, we are node %d.\n", this_node);
  for(i=1; i<m->length / sizeof(struct in_addr); i++)
    define_node(i, &n[i], i == this_node);
  DBG("Node list defined OK\n");
}

int
handle_pkt(struct slicemsg *m)
{
  switch (m->cmd)
    {
    case SLICE_CMD_DEATH_NOTIFY:
      printf("Node %d is dead.\n", m->from);
      if (!m->from)
	printf("Connection to the master is dead. Awaiting doom!\n");
      break;
    default:
      return 0;
    }
  return 1;
}

int
main(int argc, char **argv)
{
  if (argc > 1 && !strcmp(argv[1], "-d"))
    {
      argc--, argv++;
      debug = 1;
    }
  if (argc != 2)
    die("Usage: cslaved [-d] <master-node>");
  timer_period = STATUS_REPORT_PERIOD;
  daemon_init();
  connect_to_master(argv[1]);
  main_loop();
  return 0;
}
