/*
 *	Simple Linux Clustering Extensions -- Library Functions
 *
 *	Copyright (c) 1999 Martin Mares <mj@ucw.cz>
 *		  (c) 1999 Michal Jezek <mjez6230@ss1000.ms.mff.cuni.cz>
 *		  (c) 1999 Jan Lana <jlan3028@ss1000.ms.mff.cuni.cz>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>

#ifdef __GLIBC__
#include <sys/syscall.h>
#else
#include <asm/unistd.h>
#endif

#include "lib.h"

void __attribute__((noreturn))
die(char *msg, ...)
{
  va_list args;

  va_start(args, msg);
  vfprintf(stderr, msg, args);
  fputc('\n', stderr);
  exit(1);
}

void *
xmalloc(unsigned int howmuch)
{
  void *p = malloc(howmuch);
  if (!p)
    die("Unable to allocate %d bytes of memory", howmuch);
  return p;
}

#ifdef __GLIBC__

int
slicectl(int op, struct slicemsg *m)
{
  return syscall(__NR_slicectl, op, m);
}

#else

_syscall2(int, slicectl, int, op, struct slicemsg *, m);

#endif

void
pkt_dump(struct slicemsg *m)
{
  int i;
  printf("** %d -> %d  %04x [%d] ID=%x A=%x\n",
	 m->from, m->to, m->cmd, m->length, m->ident, m->arg);
  if (m->length)
    {
      for(i=0; i<m->length; i++)
	{
	  if (!(i % 16))
	    {
	      if (i)
		putchar('\n');
	      printf("\t%03x: ", i);
	    }
	  else
	    putchar(' ');
	  printf("%02x", m->data[i]);
	}
      putchar('\n');
    }
}

void
sm_send(struct slicemsg *m)
{
  if (slicectl(SLICE_OP_SEND_PACKET, m) < 0)
    die("sm_send: %m");
}
