/*
 *      Simple Linux Clustering Extensions -- The Scheduler
 *
 *      Copyright (c) 1999 Martin Mares <mj@ucw.cz>
 *                (c) 1999 Michal Jezek <mjez6230@ss1000.ms.mff.cuni.cz>
 *                (c) 1999 Jan Lana <jlan3028@ss1000.ms.mff.cuni.cz>
 *
 *      Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "lib.h"
#include "sched.h"

#define SCHED_MEM_0 10
#define SCHED_SWAP_0 10

#define SCHED_MEM_C 5.0
#define SCHED_MEM_E 2.0

#define SCHED_SWAP_C 2.0
#define SCHED_SWAP_E 5.0


static void
sched_recalc_pri(int node)
{
  struct sched_status *s = &nodes[node].sched;

  if (nodes[node].status.admit > 1)
    s->priority = s->s_prior + s->d_prior + (s->runs << FSHIFT);
  else
    s->priority = ~0;
}


int
sched_get_node(int from)
{
  int i;
  unsigned int best_p = ~0;
  int best_n = from;

  for(i=0; i<num_nodes; i++)
    if (nodes[i].sched.priority < best_p)
      {
	best_p = nodes[i].sched.priority;
	best_n = i;
      }
  nodes[best_n].sched.runs++;
  sched_recalc_pri(best_n);
  return best_n;
}


void
sched_change_stat(int node)
{
  struct slice_status *ss = &nodes[node].status;
  struct sched_status *ds = &nodes[node].sched;
  unsigned int m0, s0;
  unsigned int mem_prior, swap_prior, load_prior;
  
  m0 = ss->totalmem / SCHED_MEM_0;
  s0 = ss->totalswap / SCHED_SWAP_0;

  if (m0 < ss->freemem) 
    mem_prior = 0;
  else 
    mem_prior = (unsigned int) (SCHED_MEM_C * (1<<FSHIFT) * pow(SCHED_MEM_E, (1.0*m0)/(ss->freemem + 1.0)));	//FIXME

  if(s0 < ss->freeswap) 
    swap_prior = 0;
  else 
    swap_prior = (unsigned int) (SCHED_SWAP_C * (1<<FSHIFT) * pow(SCHED_SWAP_E, (1.0*m0)/(ss->freeswap + 1.0)));	//FIXME

  load_prior = ss->load * (1<<FSHIFT / 100);
  load_prior >>= (FSHIFT/2);
  load_prior *= load_prior;

  ds->s_prior = mem_prior + swap_prior + load_prior;

  DBG("-- mem=%d swap=%d load=%d -> static=%d\n", mem_prior, swap_prior, load_prior, ds->s_prior);
  sched_recalc_pri(node);
}
  

void
sched_tick(void)
{
  int i;
  unsigned int prior;
  
  for(i=0; i<num_nodes; i++) {
    /* the calculation take from linux/sched.h */
    prior = nodes[i].sched.d_prior;
    prior = (prior * EXP_1S + (1 << (FSHIFT-1))) >> FSHIFT;
    prior += nodes[i].sched.runs << FSHIFT;
    nodes[i].sched.d_prior = prior;
    nodes[i].sched.runs = 0;
    sched_recalc_pri(i);
  }
}


void
sched_death(int node)
{
  bzero(&nodes[node].sched, sizeof(struct sched_status));
}
